<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
        "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head><meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
<title>The Measurement Factory</title>
<style type="text/css"><!--

	div.section {
		margin-left: 2em;
	}

	div.section h2 {
		font-size: 100%;
	}

	div.section table {
		margin-left: 2em;
		background-color: #CCCCCC;
		color: black;
	}

	td a {
		text-decoration: none;
	}

	td.N, td.diff {
		text-align: right;
	}

--></style>
</head>
