
/* Web Polygraph       http://www.web-polygraph.org/
 * Copyright 2003-2011 The Measurement Factory
 * Licensed under the Apache License, Version 2.0 */

#ifndef POLYGRAPH__XSTD_H_PROCESS_H
#define POLYGRAPH__XSTD_H_PROCESS_H

#if HAVE_PROCESS_H
#include <process.h>
#endif

#endif
