
/* Web Polygraph       http://www.web-polygraph.org/
 * Copyright 2003-2011 The Measurement Factory
 * Licensed under the Apache License, Version 2.0 */

#ifndef POLYGRAPH__XSTD_XSTD_H
#define POLYGRAPH__XSTD_XSTD_H

// xstd.h must be included by all source files of xstd library

using namespace std;

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#endif
