
/* Web Polygraph       http://www.web-polygraph.org/
 * Copyright 2003-2011 The Measurement Factory
 * Licensed under the Apache License, Version 2.0 */

#ifndef POLYGRAPH__PGL_PGL_H
#define POLYGRAPH__PGL_PGL_H

using namespace std;

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "xstd/h/iostream.h"

#include "xstd/String.h"
#include "xparser/xparser.h"

#endif
