
/* Web Polygraph       http://www.web-polygraph.org/
 * Copyright 2003-2011 The Measurement Factory
 * Licensed under the Apache License, Version 2.0 */

#ifndef POLYGRAPH__BASE_MACROS_H
#define POLYGRAPH__BASE_MACROS_H

class String;

extern String ExpandMacro(const String &str, const String &macro, const String &replacement);

#endif
