
/* Web Polygraph       http://www.web-polygraph.org/
 * Copyright 2003-2011 The Measurement Factory
 * Licensed under the Apache License, Version 2.0 */

#ifndef POLYGRAPH__BASE_POLYGRAPH_H
#define POLYGRAPH__BASE_POLYGRAPH_H

// polygraph.h file must be included by all source files

using namespace std;

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "xstd/Time.h"
#include "xstd/Size.h"
#include "xstd/Assert.h"

#endif
