
/* Web Polygraph       http://www.web-polygraph.org/
 * Copyright 2003-2011 The Measurement Factory
 * Licensed under the Apache License, Version 2.0 */

#ifndef POLYGRAPH__XPARSER_XPARSER_H
#define POLYGRAPH__XPARSER_XPARSER_H

#include "config.h"

#include <stdlib.h>

#include "xstd/h/iostream.h"
#include "xstd/h/iomanip.h"

#ifndef here
#define here __FILE__ << ':' << __LINE__ << ": "
#endif

#endif
