
/* Web Polygraph       http://www.web-polygraph.org/
 * Copyright 2003-2011 The Measurement Factory
 * Licensed under the Apache License, Version 2.0 */

#ifndef POLYGRAPH__APP_DEBUGSWITCH_H
#define POLYGRAPH__APP_DEBUGSWITCH_H

// on/off signal-driven switch that controls high debugging "level"
extern void DebugSignal(int s);

#endif
