
/* Web Polygraph       http://www.web-polygraph.org/
 * Copyright 2003-2011 The Measurement Factory
 * Licensed under the Apache License, Version 2.0 */

#ifndef POLYGRAPH__XSTD_H_NETDB_H
#define POLYGRAPH__XSTD_H_NETDB_H

#ifdef HAVE_NETDB_H
#include <netdb.h>
#endif


#endif
